package com.link.core.util;

import com.link.core.basic.service.BasicServiceException;
import com.link.core.cllog.LCLogger;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;
import java.util.List;
import java.util.TimeZone;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class DateUtil {
   private static final Logger logger = LogManager.getLogger(DateUtil.class);
   public static final String DATE_TIME = "yyyy-MM-dd HH:mm:ss";
   public static final String DAY = "yyyy-MM-dd";
   public static final SimpleDateFormat DAY_FORMATTER = new SimpleDateFormat("yyyy-MM-dd");
   public static final SimpleDateFormat DATETIME_FORMATTER = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
   public static final Pattern PATTERN = Pattern.compile("^[1-2][0-9]{3}-(0[1-9]|1[0-2])-([0-2][1-9]|3[0-1])$");

   public static String dateToStr(Date date, String formatterStr) {
      if (StringUtils.isNull(formatterStr)) {
         formatterStr = "yyyy-MM-dd HH:mm:ss";
      }

      try {
         SimpleDateFormat format = new SimpleDateFormat(formatterStr);
         return format.format(date);
      } catch (Exception var3) {
         Exception e = var3;
         System.err.println("时间转换字符串错误：" + e.toString());
         return "0000-00-00 00:00:00";
      }
   }

   public static Date stringToDate(String str, String formatterStr) {
      Date date = null;

      try {
         if (StringUtils.isNull(formatterStr)) {
            formatterStr = "yyyy-MM-dd HH:mm:ss";
         }

         SimpleDateFormat format = new SimpleDateFormat(formatterStr);
         date = format.parse(str);
         return date;
      } catch (ParseException var4) {
         ParseException e = var4;
         System.err.println("字符串转换为时间错误：" + e.toString());
         return null;
      }
   }

   public static boolean isDayFormatString(String str) {
      Matcher matcher = PATTERN.matcher(str);
      return matcher.matches();
   }

   public static String unixTimetampToDateString(String unixTimetamp) {
      Long timeStamp = Long.valueOf(unixTimetamp) * 1000L;
      synchronized(DATETIME_FORMATTER) {
         return DATETIME_FORMATTER.format(new Date(timeStamp));
      }
   }

   public static Long getCurrentLongTime() {
      return System.currentTimeMillis();
   }

   public static String getLastWeekLastDay() {
      Calendar monday = Calendar.getInstance();
      monday.set(7, 2);
      monday.add(5, -1);
      Date date = new Date(monday.getTimeInMillis());
      synchronized(DAY_FORMATTER) {
         return DAY_FORMATTER.format(date);
      }
   }

   public static String getNextWeekFirstDay() {
      Calendar sunday = Calendar.getInstance();
      sunday.set(7, 2);
      sunday.add(5, 7);
      Date date = new Date(sunday.getTimeInMillis());
      synchronized(DAY_FORMATTER) {
         return DAY_FORMATTER.format(date);
      }
   }

   public static int getFirstWeekDay(String str) {
      SimpleDateFormat formatYMD = new SimpleDateFormat("yyyy-MM-dd");
      SimpleDateFormat formatD = new SimpleDateFormat("E");
      Date d = null;
      String weekDay = "";
      int n = 0;

      try {
         d = formatYMD.parse(str);
         weekDay = formatD.format(d);
         if ("星期日".equals(weekDay)) {
            n = 6;
         } else if ("星期六".equals(weekDay)) {
            n = 5;
         } else if ("星期五".equals(weekDay)) {
            n = 4;
         } else if ("星期四".equals(weekDay)) {
            n = 3;
         } else if ("星期三".equals(weekDay)) {
            n = 2;
         } else if ("星期二".equals(weekDay)) {
            n = 1;
         } else if ("星期一".equals(weekDay)) {
            n = 0;
         }
      } catch (Exception var7) {
         Exception e = var7;
         logger.error("DateUtil捕获异常: ", e);
      }

      return n;
   }

   public static Date getMonthStart(Date date) {
      if (null == date) {
         date = new Date();
      }

      Calendar calendar = Calendar.getInstance();
      calendar.setTime(date);
      int index = calendar.get(5);
      calendar.add(5, 1 - index);
      return calendar.getTime();
   }

   public static Date getMonthEnd(Date date) {
      if (null == date) {
         date = new Date();
      }

      Calendar calendar = Calendar.getInstance();
      calendar.setTime(date);
      calendar.add(2, 1);
      int index = calendar.get(5);
      calendar.add(5, -index);
      return calendar.getTime();
   }

   public static List<String> getAllDaysOfMonth(Date param) {
      List<String> list = new ArrayList();
      Date monthStart = getMonthStart(param);

      for(Date monthEnd = getMonthEnd(param); !monthStart.after(monthEnd); monthStart = getNext(monthStart)) {
         synchronized(DAY_FORMATTER) {
            list.add(DAY_FORMATTER.format(monthStart));
         }
      }

      return list;
   }

   private static Date getNext(Date date) {
      Calendar calendar = Calendar.getInstance();
      calendar.setTime(date);
      calendar.add(5, 1);
      return calendar.getTime();
   }

   public static Date addYearToDate(Date date, int year) {
      return date == null ? date : addTimes(1, date, year);
   }

   public static Date addMonthToDate(Date date, int month) {
      return date == null ? date : addTimes(2, date, month);
   }

   public static Date addDayToDate(Date date, int day) {
      return date == null ? date : addTimes(5, date, day);
   }

   public static Date addHourToDate(Date date, int hour) {
      return date == null ? date : addTimes(10, date, hour);
   }

   public static Date addMinuteToDate(Date date, int minute) {
      return date == null ? date : addTimes(12, date, minute);
   }

   public static String getCurrentYear() {
      return String.valueOf(getByScale(1));
   }

   public static String getCurrentMonth() {
      return String.valueOf(getByScale(2) + 1);
   }

   public static String getCurrentDay() {
      return String.valueOf(getByScale(5));
   }

   public static String getCurrentHour() {
      return String.valueOf(getByScale(11));
   }

   public static String getCurrentDayOfWeek() {
      return String.valueOf(getByScale(7));
   }

   public static String getCurrentDayOfMonth() {
      return String.valueOf(getByScale(5));
   }

   public static String getCurrentDayOfYear() {
      return String.valueOf(getByScale(6));
   }

   public static String getFirstDayOfWeek(String formatter) {
      if (StringUtils.isBlank(formatter)) {
         formatter = "yyyy-MM-dd";
      }

      SimpleDateFormat simpleDateFormat = new SimpleDateFormat(formatter);
      Calendar cal = Calendar.getInstance();
      int dayOfWeek = cal.get(7) - 1;
      if (dayOfWeek == 0) {
         dayOfWeek = 7;
      }

      cal.add(5, -dayOfWeek + 1);
      return simpleDateFormat.format(cal.getTime());
   }

   public static String getLastDayOfWeek(String formatter) {
      if (StringUtils.isBlank(formatter)) {
         formatter = "yyyy-MM-dd";
      }

      SimpleDateFormat simpleDateFormat = new SimpleDateFormat(formatter);
      Calendar cal = Calendar.getInstance();
      int dayOfWeek = cal.get(7) - 1;
      if (dayOfWeek == 0) {
         dayOfWeek = 7;
      }

      cal.add(5, -dayOfWeek + 7);
      return simpleDateFormat.format(cal.getTime());
   }

   public static String getFirstDayOfMonth() {
      Calendar cale = Calendar.getInstance();
      cale.add(2, 0);
      cale.set(5, 1);
      synchronized(DAY_FORMATTER) {
         return DAY_FORMATTER.format(cale.getTime());
      }
   }

   public static String getLastDayOfMonth() {
      Calendar cale = Calendar.getInstance();
      cale.add(2, 1);
      cale.set(5, 0);
      synchronized(DAY_FORMATTER) {
         return DAY_FORMATTER.format(cale.getTime());
      }
   }

   public static String getFirstDayOfLastMonth(String dateStr, String formatter) {
      if (StringUtils.isBlank(formatter)) {
         formatter = "yyyy-MM-dd";
      }

      try {
         SimpleDateFormat sdf = new SimpleDateFormat(formatter);
         Date date = sdf.parse(dateStr);
         Calendar calendar = Calendar.getInstance();
         calendar.setTime(date);
         calendar.set(5, 1);
         calendar.add(2, -1);
         return sdf.format(calendar.getTime());
      } catch (ParseException var5) {
         ParseException e = var5;
         LCLogger.withException(e);
         throw new BasicServiceException("时间转换失败：" + e.getMessage());
      }
   }

   public static String getFirstDayOfLastMonth(Date date, String formatter) {
      if (StringUtils.isBlank(formatter)) {
         formatter = "yyyy-MM-dd";
      }

      if (date == null) {
         date = new Date();
      }

      Calendar calendar = Calendar.getInstance();
      calendar.setTime(date);
      calendar.set(5, 1);
      calendar.add(2, -1);
      SimpleDateFormat sdf = new SimpleDateFormat(formatter);
      return sdf.format(calendar.getTime());
   }

   public static String getLastDayOfLastMonth(Date date, String formatter) {
      if (StringUtils.isBlank(formatter)) {
         formatter = "yyyy-MM-dd";
      }

      if (date == null) {
         date = new Date();
      }

      Calendar calendar = Calendar.getInstance();
      calendar.setTime(date);
      calendar.set(5, 1);
      calendar.add(2, -1);
      calendar.set(5, calendar.getActualMaximum(5));
      SimpleDateFormat sdf = new SimpleDateFormat(formatter);
      return sdf.format(calendar.getTime());
   }

   public static String getFirstDayOfNextMonth(String dateStr, String formatter) {
      if (StringUtils.isBlank(formatter)) {
         formatter = "yyyy-MM-dd";
      }

      try {
         SimpleDateFormat sdf = new SimpleDateFormat(formatter);
         Date date = sdf.parse(dateStr);
         Calendar calendar = Calendar.getInstance();
         calendar.setTime(date);
         calendar.set(5, 1);
         calendar.add(2, 1);
         return sdf.format(calendar.getTime());
      } catch (ParseException var5) {
         ParseException e = var5;
         LCLogger.withException(e);
         throw new BasicServiceException("时间转换失败：" + e.getMessage());
      }
   }

   public static double getDifferentDays(Date before, Date after) {
      long beforeTime = before.getTime();
      long afterTime = after.getTime();
      return (double)(afterTime - beforeTime) / 8.64E7;
   }

   public static int minutesBetween(Calendar date1, Calendar date2) {
      long time1 = date1.getTimeInMillis();
      long time2 = date2.getTimeInMillis();
      long between_days = Math.abs((time1 - time2) / 60000L);
      return Integer.parseInt(String.valueOf(between_days));
   }

   public static int daysBetween(Date smdate, Date bdate) throws ParseException {
      SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
      smdate = sdf.parse(sdf.format(smdate));
      bdate = sdf.parse(sdf.format(bdate));
      Calendar cal = Calendar.getInstance();
      cal.setTime(smdate);
      long time1 = cal.getTimeInMillis();
      cal.setTime(bdate);
      long time2 = cal.getTimeInMillis();
      long between_days = (time2 - time1) / 86400000L;
      return Integer.parseInt(String.valueOf(between_days));
   }

   public static Date addSecondToDate(Date date, int second) {
      return date == null ? date : addTimes(13, date, second);
   }

   public static Date addTimes(int field, Date date, int amount) {
      if (date == null) {
         return null;
      } else {
         Calendar cal = Calendar.getInstance();
         cal.setTime(date);
         cal.add(field, amount);
         date = cal.getTime();
         return date;
      }
   }

   public static Date attemptStringToDate(String dateStr) {
      if (dateStr.contains("T")) {
         return stringToDate(dateStr.substring(0, 10) + " " + dateStr.substring(11, 19), "yyyy-MM-dd HH:mm:ss");
      } else {
         try {
            return attempStringToDate(dateStr, "yyyy-MM-dd HH:mm:ss");
         } catch (Exception var8) {
            try {
               return attempStringToDate(dateStr, "yyyy-MM-dd HH:mm");
            } catch (Exception var7) {
               try {
                  return attempStringToDate(dateStr, "yyyy-MM-dd HH");
               } catch (Exception var6) {
                  try {
                     return attempStringToDate(dateStr, "yyyy-MM-dd");
                  } catch (Exception var5) {
                     return null;
                  }
               }
            }
         }
      }
   }

   private static Date attempStringToDate(String str, String formatterStr) throws Exception {
      SimpleDateFormat format = new SimpleDateFormat(formatterStr);
      return format.parse(str);
   }

   private static int getByScale(int scale) {
      Calendar cale = Calendar.getInstance();
      return cale.get(scale);
   }

   public static boolean isToday(Date inputDate) throws Exception {
      boolean flag = false;
      long currentTimeMillis = System.currentTimeMillis();
      Date nowDate = new Date(currentTimeMillis);
      SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
      String formatDate = simpleDateFormat.format(nowDate);
      String beginTime = formatDate + " 00:00:00";
      String endTime = formatDate + " 23:59:59";
      simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
      Date parseBeginTime = simpleDateFormat.parse(beginTime);
      Date parseEndTime = simpleDateFormat.parse(endTime);
      if (inputDate.after(parseBeginTime) && inputDate.before(parseEndTime)) {
         flag = true;
      }

      return flag;
   }

   public static String getDateTimeByTimeZone(String timezone) {
      if (StringUtils.isBlank(timezone)) {
         return dateToStr(new Date(), "yyyy-MM-dd HH:mm:ss");
      } else {
         Calendar calendar = Calendar.getInstance();
         TimeZone sourceTimeZone = calendar.getTimeZone();
         long sourceTimeMillis = calendar.getTime().getTime();
         int sourceRawOffset = sourceTimeZone.getRawOffset();
         TimeZone targetTimezone = TimeZone.getTimeZone(timezone);
         int targetRawOffset = targetTimezone.getRawOffset();
         long targetTime = sourceTimeMillis - (long)sourceRawOffset + (long)targetRawOffset;
         System.out.println("源时区与目标时区时间差: " + (sourceTimeMillis - targetTime) / 3600000L + " 小时");
         return dateToStr(new Date(targetTime), "yyyy-MM-dd HH:mm:ss");
      }
   }

   public static String getSysTimeZone() {
      return Calendar.getInstance().getTimeZone().getID();
   }

   public static Long dateToTimestamp(String time) {
      SimpleDateFormat dayFormatter = new SimpleDateFormat("yyyy-MM-dd");
      SimpleDateFormat dateTimeFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

      try {
         boolean isDay = isDayFormatString(time);
         Date date = null;
         if (isDay) {
            date = dayFormatter.parse(time);
         } else {
            date = dateTimeFormatter.parse(time);
         }

         long ts = date.getTime() / 1000L;
         return ts;
      } catch (ParseException var7) {
         throw new BasicServiceException(time + "时间戳转换失败！");
      }
   }

   public static String getFiscalYearByDateTime(Date date) {
      if (null == date) {
         date = new Date();
      }

      SimpleDateFormat df = new SimpleDateFormat("yyyy-MM");
      String publishDate = df.format(date);
      String[] strings = publishDate.split("-");
      String year = strings[0];
      String month = strings[1];
      return Integer.parseInt(month) > 10 ? String.valueOf(Integer.parseInt(year) + 1) : year;
   }

   public static List<String> getFiscalMonths(String fiscalYear) {
      List<String> result = new ArrayList();
      List<String> months = Arrays.asList("11", "12", "01", "02", "03", "04", "05", "06", "07", "08", "09", "10");
      Iterator var3 = months.iterator();

      while(var3.hasNext()) {
         String month = (String)var3.next();
         String realYear = Integer.parseInt(month) > 10 ? String.valueOf(Integer.parseInt(fiscalYear) - 1) : fiscalYear;
         String item = realYear.concat("-").concat(month);
         result.add(item);
      }

      return result;
   }

   public static Integer getTotalWeek(Date date) {
      if (date == null) {
         date = new Date();
      }

      String time = dateToStr(date, (String)null);
      String[] timeString = time.split("-");
      Calendar cal = Calendar.getInstance();
      cal.set(1, Integer.parseInt(timeString[0]));
      cal.set(2, Integer.parseInt(timeString[1]) - 1);
      cal.setFirstDayOfWeek(2);
      return cal.getActualMaximum(4);
   }
}
